// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt
//
// <Author: Christoph Kubisch>


#ifndef __LUXCORE_BITARRAY_H__
#define __LUXCORE_BITARRAY_H__

#include <luxplatform/luxplatform.h>
#include <luxcore/debug.h>
#include <memory.h>

#ifdef __cplusplus
extern "C"{
#endif

//////////////////////////////////////////////////////////////////////////
// BitArray

typedef struct BitArray_s{
	uint32*	bits;
	size_t	num32;
}BitArray_t;

// if out ouf range assert is thrown
void	BitArray_set(BitArray_t *ba, uint bit, booln state);
booln	BitArray_get(const BitArray_t *ba, uint bit);
// sets all to 0
void	BitArrray_clear(BitArray_t *ba);
// sets all to 1
void	BitArrray_all(BitArray_t *ba);
// is any bit set
booln	BitArrray_any(const BitArray_t *ba);

//////////////////////////////////////////////////////////////////////////


LUX_INLINE void	BitArray_set(BitArray_t *ba, uint bit, booln state)
{
	LUX_ASSERT(bit < ba->num32*32);
	if (state){
		ba->bits[bit/32] |= 1<<(bit%32);
	}
	else{
		ba->bits[bit/32] &= ~(1<<(bit%32));
	}
}

LUX_INLINE booln	BitArray_get(const BitArray_t *ba, uint bit)
{
	LUX_ASSERT(bit < ba->num32*32);
	return (ba->bits[bit/32] & 1<<(bit%32))!= 0;
}

LUX_INLINE void	BitArrray_clear(BitArray_t *ba)
{
	memset(ba->bits,0,sizeof(uint32)*ba->num32);
}
LUX_INLINE void	BitArrray_all(BitArray_t *ba)
{
	memset(ba->bits,0xFFFFFFFF,sizeof(uint32)*ba->num32);
}
LUX_INLINE booln	BitArrray_any(const BitArray_t *ba)
{
	const uint32 *start = ba->bits;
	const uint32 *end = start + ba->num32;

	while (start < end){
		if (start)
			return TRUE;
		start++;
	}
	return FALSE;
}

#ifdef __cplusplus
};  
#endif

#endif
