// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt
//
// <Author: Eike Decker>


#ifndef __LUXCORE_STRINGMAP_H__
#define __LUXCORE_STRINGMAP_H__

#include <luxplatform/luxplatform.h>

#ifdef __cplusplus
extern "C"{
#endif

#include <luxcore/memorygeneric.h>

////////////////////////////////////////////////////////////////////////////////
// CharStrMap
//	associative char*->void* array - usefull for string lookups (beats 	
// 	a linear stringcompare search if n>4


typedef struct CharStrMap_s*	CharStrMapPTR;

typedef void* (CharStrMapIterator_fn)(void* fnData,const char *,void *browseData);


#if defined(MEMORY_GENERIC_STATS)
	#define CharStrMap_new() CharStrMap_newTrace(__FILE__,__LINE__)
	CharStrMapPTR  CharStrMap_newTrace(const char *file, int line);
#else
	CharStrMapPTR  CharStrMap_new();
#endif

// returns 1 if something was added, or 0 when key was overwritten
booln	CharStrMap_set(CharStrMapPTR	self, const char *key, void *value);
void*	CharStrMap_get(CharStrMapPTR	self, const char *key);
void*	CharStrMap_getNth(CharStrMapPTR	self, uint n);
uint	CharStrMap_getCount(CharStrMapPTR	self);

// returns 1 on success
booln	CharStrMap_unset(CharStrMapPTR	self, const char *key);
booln	CharStrMap_isset(CharStrMapPTR	self, const char *key);
void	CharStrMap_free(CharStrMapPTR		self, void (*fnFree)(void*));

// iterator returns fnData for next call
void	CharStrMap_iterate(CharStrMapPTR	self, CharStrMapIterator_fn *itfunc, void *fnData);

#ifdef __cplusplus
}
#endif

#endif

