// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt
//
// <Author: Christoph Kubisch>
// modelled after Yasli:Vector


#ifndef __LUXCORE_CONTVECTOR_H__
#define __LUXCORE_CONTVECTOR_H__

#include <luxplatform/luxplatform.h>
#include <luxcore/debug.h>

#ifdef __cplusplus
extern "C"{
#endif

	#define CONTVECTOR_SMALL	(8)

	typedef struct ContVector_s{
		size_t		elemsize;
		size_t		alignsize;
		
		byte*		beg;
		byte*		end;
		byte*		eos;
	}ContVector_t;

	void ContVector_init(ContVector_t* cv, size_t elemsize);

	// alignsize must be power of 2 and should be >= 4
	void ContVector_initAligned(ContVector_t* cv, size_t elemsize, size_t alignsize);
	ContVector_t ContVector_new(size_t elemsize);
	ContVector_t ContVector_copy(ContVector_t* cv);

	void*	ContVector_front(ContVector_t* cv);
	void*	ContVector_back(ContVector_t* cv);
	void*	ContVector_at(ContVector_t* cv, size_t idx);

	size_t	ContVector_elemsize(ContVector_t *cv);
	size_t	ContVector_capacity(ContVector_t *cv);
	size_t	ContVector_size(ContVector_t *cv);
	size_t	ContVector_fullsize(ContVector_t *cv);
	booln	ContVector_empty(ContVector_t *cv);

	void	ContVector_pushBack(ContVector_t *cv, const void *data);
	void	ContVector_popBack(ContVector_t *cv);
	void	ContVector_clear(ContVector_t *cv);

	void	ContVector_insert(ContVector_t *cv, size_t idx, const void *data);
	void	ContVector_insertRepeat(ContVector_t *cv, size_t idx, const void *data, size_t cnt);
	void	ContVector_insertMany(ContVector_t *cv, size_t idx, const void *data, size_t cnt);
	void	ContVector_remove(ContVector_t *cv, size_t idx);
	void	ContVector_removeMany(ContVector_t *cv, size_t idx, size_t cnt);
	
	void	ContVector_shrink(ContVector_t *cv);
	void	ContVector_reserve(ContVector_t *cv,size_t cnt);
	void	ContVector_resize(ContVector_t *cv,size_t cnt, const void *fill);



	LUX_INLINE void ContVector_init(ContVector_t* cv, size_t elemsize){
		cv->elemsize = elemsize;
		cv->alignsize = 0;
		cv->end = cv->beg;
		cv->eos = cv->beg;
	}
	LUX_INLINE void ContVector_initAligned(ContVector_t* cv, size_t elemsize, size_t alignsize){
		cv->elemsize = elemsize;
		cv->alignsize = alignsize;
		cv->end = cv->beg;
		cv->eos = cv->beg;
	}
	LUX_INLINE ContVector_t ContVector_new(size_t elemsize){
		ContVector_t	cv;
		cv.elemsize = elemsize;
#ifdef _DEBUG
		cv.beg = NULL;
		cv.end = NULL;
		cv.eos = NULL;
#else
		cv.end = cv.beg;
		cv.eos = cv.beg;
#endif

		return cv;
	}

	LUX_INLINE size_t	ContVector_elemsize(ContVector_t *cv){
		return cv->elemsize;
	}
	LUX_INLINE size_t	ContVector_fullsize(ContVector_t *cv){
		return (cv->end-cv->beg);
	}
	LUX_INLINE size_t	ContVector_capacity(ContVector_t *cv){
		return (cv->eos-cv->beg)/cv->elemsize;
	}
	LUX_INLINE size_t	ContVector_size(ContVector_t *cv){
		return (cv->end-cv->beg)/cv->elemsize;
	}
	LUX_INLINE booln	ContVector_empty(ContVector_t *cv){
		return cv->beg == cv->end;
	}

	LUX_INLINE void*	ContVector_front(ContVector_t* cv){
		LUX_ASSERT(!ContVector_empty(cv));
		return (void*)cv->beg;
	}
	LUX_INLINE void*	ContVector_back(ContVector_t* cv){
		LUX_ASSERT(!ContVector_empty(cv));
		return (void*)(cv->end-cv->elemsize);
	}
	LUX_INLINE void*	ContVector_at(ContVector_t* cv, size_t idx){
		LUX_ASSERT(!ContVector_empty(cv) && idx < ContVector_size(cv));
		return (void*)(cv->beg+(cv->elemsize*idx));
	}

#ifdef __cplusplus
}
#endif


#endif
