// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt
//
// <Author: Christoph Kubisch>


#ifndef __LUXCORE_MEMORYBASE_H__
#define __LUXCORE_MEMORYBASE_H__

#include <luxplatform/luxplatform.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
//////////////////////////////////////////////////////////////////////////
// Switches
//
//	MEMORY_GENERIC_STATS
//
//	MEMORY_DEFAULT_DLMALLOC

#ifdef __cplusplus
extern "C"{
#endif

#define MEMORY_MEGS(a)		(a*1048576)
#define MEMORY_KBS(a)		(a*1024)

	typedef struct MemAllocator_s{
		void*	(__cdecl *_malloc)(size_t);
		void*	(__cdecl *_calloc)(size_t,size_t);
		void*	(__cdecl *_realloc)(void*,size_t);
		void	(__cdecl *_free)(void*);
	}MemAllocator_t;

	LUX_API extern MemAllocator_t	g_MemAllocator;

#define MEMORY_GLOBAL_MALLOC		g_MemAllocator._malloc
#define MEMORY_GLOBAL_CALLOC		g_MemAllocator._calloc
#define MEMORY_GLOBAL_REALLOC		g_MemAllocator._realloc
#define MEMORY_GLOBAL_FREE			g_MemAllocator._free

//////////////////////////////////////////////////////////////////////////
// Aligned alloc & free

LUX_API void	alignedFree(void *ptr);

// alignsize must be power of 2
LUX_API void*	alignedMalloc(size_t size, size_t align_size);

void*	alignedToOrig(void *ptr);
void*	alignedFromOrig(void *ptr, size_t align_size);


//////////////////////////////////////////////////////////////////////////
// Pointer

// advances a pointer for continous allocation, assumes all allocations
// fit into memory block
LUX_API void*	pointerAdvanceAlloc(void **ptr, size_t size);
LUX_API void*	pointerAdvanceAllocAligned(void **ptr, size_t size, size_t align);

// aligns must be power of 2
void*	pointerAlign(void *ptr, size_t align);

//////////////////////////////////////////////////////////////////////////
LUX_INLINE void*	pointerAlign(void *ptr, size_t align){
	align = align-1;
	return (void*) ((((size_t)ptr) + align) & ~align);
}
LUX_INLINE void*	alignedToOrig(void *ptr){
	char *ptrb = (char*)ptr;
	size_t *ptr2=(size_t *)ptr - 1;
	ptrb -= *ptr2;

	return ptrb;
}

LUX_INLINE void*	alignedFromOrig(void* optr, size_t align_size)
{
	size_t align_mask = align_size - 1;
	char *ptr = (char*)optr;
	char *ptr2 = ptr + sizeof(size_t);
	char *aligned_ptr = ptr2 + (align_size - ((size_t)ptr2 & align_mask));


	ptr2 = aligned_ptr - sizeof(size_t);
	*((size_t *)ptr2)=(size_t)(aligned_ptr - ptr);

	return (void*)aligned_ptr;
}

LUX_INLINE void	alignedCorrectRealloc(void* reallocated, size_t alignsize, 
									  size_t newsize, size_t info, size_t shiftold)
{
	byte*	oldptr = (byte*)reallocated;
	byte*	ptr = (byte*)pointerAlign(oldptr+info,alignsize);
	size_t shiftnew = (ptr-oldptr);
	ptrdiff_t diff = shiftold-shiftnew;

	if (diff == 0){
		return;
	}
	memmove(ptr,oldptr+(shiftold+info),newsize);
}

#ifdef __cplusplus
};  
#endif

#endif

