// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt
//
// <Author: Christoph Kubisch, Eike Decker>


#ifndef __LUXCORE_MEMORYGEN_H__
#define __LUXCORE_MEMORYGEN_H__


#include "memorybase.h"

#ifdef _DEBUG
#define MEMORY_GENERIC_STATS
#endif


#ifdef __cplusplus
extern "C"{
#endif

//////////////////////////////////////////////////////////////////////////
// MemoryGeneric
// generic allocator, mostly for profiling/ debugging/ stat tracking
// NOT THREADSAFE!!
// zalloc = set to 0

typedef struct MemoryGenericInfo_s{
	ptrdiff_t	allocs;
	ptrdiff_t	mem;
}MemoryGenericInfo_t;

LUX_API MemoryGenericInfo_t	MemoryGeneric_getInfo();
// returns FALSE on error
LUX_API booln	MemoryGeneric_deinitStats();

LUX_API void*	MemoryGeneric_zalloc(size_t size);
LUX_API void*	MemoryGeneric_malloc(size_t size);
LUX_API void*	MemoryGeneric_calloc(size_t num, size_t size);
LUX_API void	MemoryGeneric_free(void *ptr);
LUX_API void*	MemoryGeneric_realloc(void* ptr, size_t size);
LUX_API void*	MemoryGeneric_zallocAligned(size_t size, size_t alignsize);
LUX_API void*	MemoryGeneric_reallocAligned(void* ptr,size_t size,size_t alignsize);
LUX_API void	MemoryGeneric_freeAligned(void *ptr);
LUX_API size_t	MemoryGeneric_offsetAligned(void *ptr);

#if defined(MEMORY_GENERIC_STATS)
LUX_API void*	MemoryGeneric_zallocStats(size_t size,const char *source,int line);
LUX_API void*	MemoryGeneric_mallocStats(size_t size,const char *source,int line);
LUX_API void*	MemoryGeneric_callocStats(size_t num, size_t size,const char *source,int line);
LUX_API void*	MemoryGeneric_reallocStats(void *ptr, size_t size, size_t oldsize, const char *source, int line);
LUX_API void	MemoryGeneric_freeStats(void *ptr, size_t size,const char *source,int line);
LUX_API void*	MemoryGeneric_zallocAlignedStats(size_t size, size_t alignsize,const char *source,int line);
LUX_API void*	MemoryGeneric_reallocAlignedStats(void* ptr,size_t size,size_t alignsize,size_t oldsize,const char *source,int line);
LUX_API void	MemoryGeneric_freeAlignedStats(void *ptr, size_t size,const char *source,int line);
LUX_API size_t	MemoryGeneric_offsetAlignedStats(void *ptr);

#endif




//////////////////////////////////////////////////////////////////////////

#if defined(MEMORY_GENERIC_STATS)
	#define genzalloc(size)					MemoryGeneric_zallocStats(size,__FILE__,__LINE__)
	#define genmalloc(size)					MemoryGeneric_mallocStats(size,__FILE__,__LINE__)
	#define gencalloc(size,cnt)				MemoryGeneric_callocStats(size,cnt,__FILE__,__LINE__)
	#define genrealloc(ptr,size,oldsize)	MemoryGeneric_reallocStats(ptr,size,oldsize,__FILE__,__LINE__)
	#define genfree(ptr,size)				MemoryGeneric_freeStats(ptr,size,__FILE__,__LINE__)

	#define genreallocaligned(ptr,size,aligned,oldsize)	MemoryGeneric_reallocAlignedStats(ptr,size,aligned,oldsize,__FILE__,__LINE__)
	#define genzallocaligned(size,aligned)	MemoryGeneric_zallocAlignedStats(size,aligned,__FILE__,__LINE__)
	#define genfreealigned(ptr,size)		MemoryGeneric_freeAlignedStats(ptr,size,__FILE__,__LINE__)
	#define genoffaligned(ptr)				MemoryGeneric_offsetAlignedStats(ptr)
#else
	#define genzalloc(a)					MemoryGeneric_zalloc(a)
	#define genmalloc(a)					MemoryGeneric_malloc(a)
	#define gencalloc(a,b)					MemoryGeneric_calloc(a,b)
	#define genrealloc(ptr,size,oldsize)	MemoryGeneric_realloc(ptr,size)
	#define genfree(ptr,size)				MemoryGeneric_free(ptr)

	#define genreallocaligned(ptr,size,aligned,oldsize)	MemoryGeneric_reallocAligned(ptr,size,aligned)
	#define genzallocaligned(a,aligned)		MemoryGeneric_zallocAligned(a,aligned)
	#define genfreealigned(ptr,size)		MemoryGeneric_freeAligned(ptr)
	#define genoffaligned(ptr)				MemoryGeneric_offsetAligned(ptr)
#endif



//////////////////////////////////////////////////////////////////////////

// dont access!
extern	MemoryGenericInfo_t	g_CoreMemGen;

LUX_INLINE void MemoryGeneric_free(void *ptr) {
	LUX_PROFILING_OP (g_CoreMemGen.allocs--);
	MEMORY_GLOBAL_FREE(ptr);
}

LUX_INLINE void *MemoryGeneric_zalloc(size_t size)
{
	LUX_PROFILING_OP (g_CoreMemGen.allocs++);

	{
		void *ptr = MEMORY_GLOBAL_CALLOC(1,size);

		return ptr;
	}
}

LUX_INLINE void *MemoryGeneric_calloc(size_t num, size_t size)
{
	LUX_PROFILING_OP (g_CoreMemGen.allocs++);

	{
		void *ptr = MEMORY_GLOBAL_CALLOC(num,size);

		return ptr;
	}
}

LUX_INLINE void *MemoryGeneric_malloc(size_t size)
{
	LUX_PROFILING_OP (g_CoreMemGen.allocs++);

	{
		void *ptr = MEMORY_GLOBAL_MALLOC(size);

		return ptr;
	}
}

LUX_INLINE void *MemoryGeneric_realloc(void* ptr, size_t size)
{
	ptr = MEMORY_GLOBAL_REALLOC(ptr,size);

	return ptr;
}

LUX_INLINE void MemoryGeneric_freeAligned(void *ptr) {
	//LUX_PROFILING_OP (g_CoreMemGen.mem-= (ptrdiff_t)size);
	LUX_PROFILING_OP (g_CoreMemGen.allocs--);
	ptr = alignedToOrig(ptr);
	MEMORY_GLOBAL_FREE(ptr);
}

LUX_INLINE void *MemoryGeneric_zallocAligned(size_t size, size_t alignsize)
{
	void *ptr = MEMORY_GLOBAL_MALLOC(size + alignsize + sizeof(size_t));
	
	ptr = alignedFromOrig(ptr,alignsize);
	memset(ptr,0,size);

	LUX_PROFILING_OP (g_CoreMemGen.allocs++);

	return ptr;
}

LUX_INLINE void *MemoryGeneric_reallocAligned(void* ptr, size_t newsize, size_t alignsize)
{
	byte* oldptr = alignedToOrig(ptr);
	size_t shiftold = (((byte*)ptr)-oldptr)-sizeof(size_t);

	ptr = MEMORY_GLOBAL_REALLOC(ptr,newsize + alignsize + sizeof(size_t));
	alignedCorrectRealloc(ptr,alignsize,newsize,sizeof(size_t),shiftold);

	return alignedFromOrig(ptr,alignsize);
}

LUX_INLINE size_t MemoryGeneric_offsetAligned(void *ptr)
{
	byte*	inptr = (byte*)ptr;
	byte*	optr  = (byte*)alignedToOrig(ptr);

	return (inptr-optr);
}

#ifdef __cplusplus
}
#endif

#endif

