// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt
//
// <Author: Christoph Kubisch>


#ifndef __LUXCORE_MEMORYPOOL_H__
#define __LUXCORE_MEMORYPOOL_H__

#include "memorybase.h"

#ifdef __cplusplus
extern "C"{
#endif

// will make all mempool allocs 4byte aligned
#define LUX_MEMORY_POOL_PAD4

//////////////////////////////////////////////////////////////////////////
// MemoryPool
//
// Hunks big chunks that are filled linearly
// Not able to free parts of it, just the complete chunk
// NOT THREADSAFE!!

typedef struct MemoryPool_s* MemoryPoolPTR;


LUX_API MemoryPoolPTR	MemoryPool_new(const char* name, size_t totalbytes);

// performs memset 0
LUX_API void*	MemoryPool_zalloc(MemoryPoolPTR	mem,size_t size);		
// allocs aligned + memset 0
LUX_API void*	MemoryPool_zallocAligned(MemoryPoolPTR	mem,size_t size,size_t align);
// performs no memset 0
LUX_API void*	MemoryPool_malloc(MemoryPoolPTR	mem, size_t size);
// returns current inuse pointer
LUX_API void*	MemoryPool_current(MemoryPoolPTR	mem);

// inits to size, or resizes if current is smaller
// returns TRUE if resize/new alloc happened
// also sets resized flag to true
LUX_API int		MemoryPool_initMin(MemoryPoolPTR	mem, size_t totalbytes);	

// returns resized flag and clears it
LUX_API int		MemoryPool_popResized(MemoryPoolPTR	mem);
//		sets used megs back to 0
LUX_API void	MemoryPool_clear(MemoryPoolPTR	mem);
// frees pool (dont forget !!)
LUX_API void	MemoryPool_free(MemoryPoolPTR	mem);
// returns true if ptr is inside pool, otherwise assert
LUX_API booln	 MemoryPool_checkPtr(MemoryPoolPTR	mem, void *ptr);
// from this adress on new allocs will be done
LUX_API void	MemoryPool_setBegin(MemoryPoolPTR	mem, void *ptr);
LUX_API size_t	MemoryPool_bytesLeft(MemoryPoolPTR	mem);
LUX_API size_t	MemoryPool_bytesUsed(MemoryPoolPTR	mem);
LUX_API size_t	MemoryPool_bytesTotal(MemoryPoolPTR	mem);

#ifdef __cplusplus
}
#endif

#endif
