// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt
//
// <Author: Christoph Kubisch>


#ifndef __LUXCORE_SCALARMISC_H__
#define __LUXCORE_SCALARMISC_H__

#ifdef __cplusplus
extern "C"{
#endif

typedef enum ScalarType_e{	
	// warning may not change order!!
	// interpolation is based on abs(max)
	SCALAR_FLOAT32,		// sat: [0,1]
	SCALAR_INT8,		// sat: [-127,127]
	SCALAR_UINT8,		// sat: [0,255]								
	SCALAR_INT16,		// sat: [-SHORT_SIGNEDMAX,SHORT_SIGNEDMAX]
	SCALAR_UINT16,		// sat: [0,SHORT_UNSIGNEDMAX]
	SCALAR_INT32,		// sat: same as INT16
	SCALAR_UINT32,		// sat: same as UINT16
	SCALARS,

	// only supported in getSize
	SCALAR_FLOAT16,		
	SCALAR_FLOAT64,

	SCALAR_ILLEGAL,
}ScalarType_t;

typedef union ScalarPtr_u{
	void*			tvoid;
	float*			tfloat;
	int8*			tint8;
	uint8*			tuint8;
	int16*			tint16;
	uint16*			tuint16;
	int32*			tint32;
	uint32*			tuint32;
}ScalarPtr_t;

typedef union ScalarVector_u{
	float	tfloat[4];
	uint8	tuint8[4];
	int8	tint8[4];
	uint16	tuint16[4];
	int16	tint16[4];
	uint32	tuint32[4];
	int32	tint32[4];
}ScalarVector_t;

size_t ScalarType_getSize(ScalarType_t type);

//////////////////////////////////////////////////////////////////////////
// Conversion
LUX_API void ScalarType_toFloat(float* pout, ScalarType_t intype, const void *pin, uint vectordim);
LUX_API void ScalarType_fromFloat(void* pout, ScalarType_t outtype, const float *pin, uint vectordim);

// types are interpreted as 0,1 or -1,1
LUX_API void ScalarType_toFloatNormalized(float* pout, ScalarType_t intype, void *pin, uint vectordim);
LUX_API void ScalarType_fromFloatNormalized(void* pout, ScalarType_t outtype, const float *pin, uint vectordim);
// input already is float but in original dimensions
LUX_API void ScalarType_normalizedFloat(float* pout, ScalarType_t intype, float *pin, uint vectordim);

// from/to 32bit natives ie float32 and int32, uint32 will be clamped to INT_MAX
LUX_API void ScalarType_from32(ScalarVector_t *pout, ScalarType_t type, void *pin, uint vectordim);
LUX_API void ScalarType_to32(ScalarVector_t *pout, ScalarType_t type, void *pin, uint vectordim);


//////////////////////////////////////////////////////////////////////////

LUX_INLINE size_t ScalarType_getSize(ScalarType_t type)
{
	static const size_t sizes[SCALARS+3] = {
		sizeof(float),
		sizeof(int8),
		sizeof(uint8),
		sizeof(int16),
		sizeof(uint16),
		sizeof(int32),
		sizeof(uint32),

		0,

		sizeof(ushort),
		sizeof(double),
	};

	LUX_ASSUME(type >= 0 && type < SCALARS+3);
	return sizes[type];
}

#ifdef __cplusplus
}
#endif

#endif

