// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt

#ifndef __LUXCORE_STRMISC_H__
#define __LUXCORE_STRMISC_H__

#include <luxplatform/luxplatform.h>
#include <string.h>

#ifdef __cplusplus
extern "C"{
#endif

#define STR_FORMAT_VECTOR3 "(%f,%f,%f)"
#define STR_FORMAT_VECTOR4 "(%f,%f,%f,%f)"

//////////////////////////////////////////////////////////////////////////
// string helpers

LUX_API void strPrintf(char *buffer, size_t buffersize, const char *fmt, ...);
LUX_API int	 strBeginsWith(const char *check, const char *search);
LUX_API int	 strSplitList( char *liststart,  char **items, const char sep,int maxitems);

	// replace string must be shorter or equal search string, tail char is used to pad shoter ones
LUX_API int	strReplace(char *str,const char *search,const char *replace,const char trail);

	// output must be same size as input
	// output will always be a valid string
LUX_API char* strToLower(char* output, const char *input);

	// output must be same size as input
	// output will always be a valid string
LUX_API char* strToUpper(char* output, const char *input);

	// out must be at least filename size
LUX_API char*	strGetPath(char *out,const char *filename);

	// returns formatted integer eg. 1,000 
LUX_API char*	strFormattedNumber(char *buffer,int number,int sep);

	// returns end of quote in inStr
LUX_API const char*	strReadInQuotes(const char *inStr, char *outStr, size_t outSize);
LUX_API int		strReadBitMask(const char* str, enum32 *val);
LUX_API int		strReadVector3(const char* str, float vec[3]);
LUX_API int		strReadVector4(const char* str, float vec[4]);
LUX_API int		strReadFloat(const char* str, float *val);
LUX_API int		strReadInt(const char* str, int *val);

LUX_API booln	strHasPath(const char *str);

#ifdef __cplusplus
}
#endif

#endif

