// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt

#include <luxcore/file.h>
#include <stdio.h>
#include <string.h>

char FS_getc(FSFile_t *file){
	char c = file->contents[file->index];
	if (file->index >= file->size)
		return EOF;
	file->index++;
	return c;
}

size_t FS_read(void* target, size_t size, size_t count,FSFile_t *file){
	size_t read;
	// find out how many we can copy
	if (file->size < file->index)
		return 0;
	else if ((unsigned long)(file->size - file->index) >= size*count)
		read = (size*count);
	else
		read = (size*count)%((file->size)-(file->index));
	read /= size;

	memcpy(target,&file->contents[file->index],read*size);
	file->index+=read*size;
	return read;
}

size_t FS_seek(FSFile_t* file, size_t offset, size_t origin)
{
	long oldoffset = file->index;

	switch(origin)
	{
	case SEEK_SET:
		file->index = offset;
		break;

	case SEEK_CUR:
		file->index += offset;
		break;

	case SEEK_END:
		file->index = file->size - offset;
		break;

	default:
		return FALSE;
	}

	if(offset < 0 || offset > file->size)
	{
		file->index = oldoffset;
		return FALSE;
	}

	return TRUE;
}

char * FS_gets(char *buffer, size_t size, FSFile_t *file)
{
	size_t i;
	char *string = (char*)&file->contents[file->index];

	for(i=0; i < size && i+file->index < file->size; i++)
	{
		buffer[i] = string[i];
		if(string[i] == '\0' || string[i] == '\n')
		{
			i++;
			break;
		}
	}

	buffer[i] = '\0';

	file->index += i;

	if(i == 0)
		return NULL;
	else
		return buffer;
}

char FS_eof(FSFile_t *fsfile){
	if (!fsfile)
		return TRUE;
	return fsfile->index >= fsfile->size;
}

void*	FS_getCurrent(FSFile_t *file)
{
	if(!file || file->index >= file->size) return NULL;

	return &file->contents[file->index];
}

booln	FS_setCurrent(FSFile_t *file, void* cur)
{
	byte *data = (byte*)cur;
	if (data < file->contents || data > file->contents+file->size)
		return FALSE;

	file->index = data-file->contents;

	return TRUE;
}

void *FS_getContent(FSFile_t *file)
{
	if(!file) return NULL;

	return file->contents;
}


size_t FS_getSize(FSFile_t *file)
{
	if(!file) return 0;

	return file->size;
}
