// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt

#include <luxcore/memorylist.h>
#include <luxcore/debug.h>

void MemoryList_init (MemoryList_t* self)
{
	int i;

	memset(self,0,sizeof(MemoryList_t));

	self->cnt = 16;
	self->sizes = (size_t*)MEMORY_GLOBAL_MALLOC(sizeof(int)*self->cnt);
	self->freeblocks = (size_t*) MEMORY_GLOBAL_MALLOC(sizeof(int)*self->cnt);
	self->freelists = (MemoryBlock_t**)
		MEMORY_GLOBAL_MALLOC(sizeof(MemoryBlock_t*)*self->cnt);
	for (i=0;i<self->cnt;i++)
		self->sizes[i] = 1<<(i+5);
	self->chunksize = 1<<20;
	self->chunk = (char*)MEMORY_GLOBAL_MALLOC(self->chunksize);
	self->chunkpos = 0;
	for (i=0;i<self->cnt;i++) self->freelists[i] = NULL;
	for (i=0;i<self->cnt;i++) self->freeblocks[i] = 0;
}

void MemoryList_clear (MemoryList_t* mc)
{
	// TODO
	LUX_ASSERT(0);
}

void MemoryList_freeItem (MemoryList_t* self, void* old, size_t oldsize)
{
	int i;
	MemoryBlock_t *block = (MemoryBlock_t*)old;

	if (old==NULL || oldsize==0) return;

	for (i=0;i<self->cnt;i++) {
		if (oldsize<=self->sizes[i]) {
			self->freeblocks[i]++;
			block->next = self->freelists[i];
			self->freelists[i] = block;
			return;
		}
	}

	MEMORY_GLOBAL_FREE(old);
	LUX_PROFILING_OP( self->memAllocated -= oldsize);
}

void* MemoryList_allocItem (MemoryList_t* self, size_t size)
{
	char *mem;
	int i;

	if (size > self->sizes[self->cnt-1]) {
		LUX_PROFILING_OP( self->memAllocated += size*sizeof(char));
		return MEMORY_GLOBAL_MALLOC(size*sizeof(char));
	}
	if (self->chunk==0 || self->chunkpos+size>=self->chunksize) {
		size_t nsize = self->chunk == 0 ? (1<<8) : (self->chunksize*2);
		nsize = nsize<size*2?size*2:nsize;
		LUX_PROFILING_OP( self->memAllocated += nsize*sizeof(char));
		self->chunk = (char*)MEMORY_GLOBAL_MALLOC(nsize*sizeof(char));

		LUX_DEBUGASSERT(self->chunk);
		self->chunksize = nsize;
		self->chunkpos = 0;
	}
	for (i=0;i<self->cnt;i++) {
		if (size<=self->sizes[i]) {
			size = self->sizes[i];
			break;
		}
	}
	mem = &self->chunk[self->chunkpos];
	self->chunkpos+=size;



	return mem;
}

void* MemoryList_reallocItem (MemoryList_t* self, void* old, size_t oldsize, size_t newsize)
{
	void *ptr;
	int i;

	for (i=1;i<self->cnt;i++) {
		if (oldsize <= self->sizes[i] && newsize <= self->sizes[i] &&
			oldsize > self->sizes[i-1] && newsize > self->sizes[i-1])
			return old;
	}
	for (i=0;i<self->cnt;i++) {
		if (newsize<=self->sizes[i]) {
			if (self->freeblocks[i]==0) {
				ptr = (void*)MemoryList_allocItem(self,self->sizes[i]);
				break;
			}
			self->freeblocks[i]--;
			ptr = (void*)self->freelists[i];
			self->freelists[i] = self->freelists[i]->next;
			break;
		}
	}

	if (i==self->cnt) {
		ptr = (void*)MemoryList_allocItem(self,newsize);
	}

	if (old) {
		memcpy(ptr,old,MIN(oldsize,newsize));
		MemoryList_freeItem(self,old,oldsize);
	}
	return ptr;
}
