// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt

#include <luxcore/strmisc.h>
#include <stdarg.h>
#include <ctype.h>

void strPrintf(char *buffer, size_t buffersize, const char *fmt, ...)
{
	va_list		ap;

	if (fmt == NULL) return;
	va_start(ap, fmt);
#ifdef LUX_COMPILER_MSC
		_vsnprintf(buffer, buffersize, fmt, ap);
#elif defined (LUX_COMPILER_GCC)
		vsnprintf(buffer, buffersize, fmt, ap);
#else
	#error "compiler unsupported"
#endif
	va_end(ap);

	buffer[buffersize-1] = '\0';
}

const char*	strReadInQuotes(const char *inStr, char *outStr, size_t outSize)
{
	const char *in = inStr;
	char test;
	char *outEnd = outStr+outSize;
	memset(outStr, 0, outSize);

	while ((test=*in) && test != '\"')in++;
	if (!test)
		return in;
	in++;
	while ((test=*in) && test != '\"' && outStr < outEnd)
	{
		*outStr++ = test;
		in++;
	}
	if (*in == '\"')
		return (++in);
	else
		return NULL;
}

int		strReadBitMask(const char* str, enum32 *val){
	int i = 0;
	*val = 0;

	while (str[i] && i < 32){
		if (str[i] == '1') 
			*val |= 1<<i++; 
		else if (str[i] == '0')
			i++;
		else
			return i;
			
	}
	return i;
}
int		strReadVector3(const char* str, float vec[3]){
	return sscanf(str,STR_FORMAT_VECTOR3,&vec[0],&vec[1],&vec[2]);
}
int		strReadVector4(const char* str, float vec[4]){
	return sscanf(str,STR_FORMAT_VECTOR4,&vec[0],&vec[1],&vec[2],&vec[3]);
}
int		strReadFloat(const char* str, float *val){
	return sscanf(str,"%f",val);
}
int		strReadInt(const char* str, int *val){
	return sscanf(str,"%i",val);
}

booln	strHasPath(const char *str){
	const char *search = str;
	while (*search != 0){
		if (*search == '/' || *search == '\\'){
			return TRUE;
		}
		search++;
	}

	return FALSE;
}

int	strBeginsWith(const char *check, const char *search)
{
	const char *src = check;
	const char *id = search;

	if (!src || !id)
		return FALSE;

	while(*src == *id){
		src++;
		id++;
	}

	if (*id )
		return FALSE;
	else
		return TRUE;
}

int	strSplitList( char *letter,  char **items, const char sep,int numtex)
{
	int tex;

	tex = 0;
	while (*letter && tex < numtex){
		items[tex] = letter;
		while (*letter && *letter != sep){
			letter++;
		}
		*letter = 0;
		letter++;
		tex++;
	}
	return (numtex < tex) ? numtex : tex;
}

int	strReplace(char *str,const char *search,const char *replace,const char trail)
{
	char *curchar;
	const char *repchar;
	int	searchlen = strlen(search);
	int	replen = strlen(replace);
	int	i;
	int cnt = 0;

	while(curchar = strstr(str,search)){
		repchar = replace;
		cnt++;
		for(i = 0; i < searchlen; i++,curchar++,repchar++){
			if (i < replen)
				*curchar = *repchar;
			else
				*curchar = trail;
		}
	}

	return cnt;
}

char*	strGetPath(char *out,const char *filename)
{
	const char	*browse = filename;
	char *lastslash = NULL;
	char	*pwrite = out;

	out[0] = '\0';

	while (browse && *browse && *browse != '.'){
		if (*browse == '/' || *browse == '\\')
			lastslash = pwrite;
		*pwrite = *browse;
		browse++;
		pwrite++;
	}

	if (lastslash){
		lastslash++;
		*lastslash = 0;
	}

	return out;
}

char*	strFormattedNumber(char *buffer,int number,int sep)
{
	char *ptr;
	switch(sep)
	{
	case 0:
		sprintf(buffer,"%3d",number);
		break;
	case 1:
		if (number > 1000)
			sprintf(buffer,"%3d,%3d",number/1000,number%1000);
		else 
			sprintf(buffer,"%7d",number);
		break;
	case 2:
		if (number > 1000000)
			sprintf(buffer,"%3d,%3d,%3d",number/1000000,(number%1000000)/1000,(number%1000000)%1000);
		else if (number > 1000)
			sprintf(buffer,"%7d,%3d",number/1000,number%1000);
		else 
			sprintf(buffer,"%11d",number);
		break;
	case 3:
		if (number > 1000000000)
			sprintf(buffer,"%3d,%3d,%3d,%3d",number/1000000000,(number%1000000000)/1000000,((number%1000000000)%1000000)/1000,((number%1000000000)%1000000)%1000);
		else if (number > 1000000)
			sprintf(buffer,"%7d,%3d,%3d",number/1000000,(number%1000000)/1000,(number%1000000)%1000);
		else if (number > 1000)
			sprintf(buffer,"%11d,%3d",number/1000,number%1000);
		else 
			sprintf(buffer,"%15d",number);
		break;
	}
	ptr = buffer+1;
	while (*ptr){
		if(*ptr == ' ' && (*(ptr-1) == ',' || *(ptr-1) == '0' ))
			*ptr = '0';
		ptr++;
	}

	return buffer;
}

char* strToLower(char* output, const char *input)
{
	const char *c = input;
	char *o = output;

	if (!input || input[0] == 0){
		output[0] = 0;
		return output;
	}

	while (c[0]){
		o[0] = tolower(c[0]);
		o++;
		c++;
	}
	o[0] = 0;

	return output;
}

char* strToUpper(char* output, const char *input)
{
	const char *c = input;
	char *o = output;

	if (!input || input[0] == 0){
		output[0] = 0;
		return output;
	}

	while (c[0]){
		o[0] = toupper(c[0]);
		o++;
		c++;
	}
	o[0] = 0;

	return output;
}
