// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt

#ifdef __cplusplus
extern "C"{
#endif

#include <luxcore/luxcore.h>
#include <luxmath/luxmath.h>

#ifdef __cplusplus
}
#endif

#ifdef LUX_PLATFORM_WINDOWS
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>


static double l_timerRes;

long initTimer(){
	__int64 freq;
	long works = QueryPerformanceFrequency( (LARGE_INTEGER *)&freq);
	LUX_ASSERT(works);
	l_timerRes = 1.0 / (double)freq;

	return works;
}

double getMicros(){
	__int64 t_64;
	QueryPerformanceCounter( (LARGE_INTEGER *)&t_64 );
	return (double)(t_64) * l_timerRes;
}

#endif


void	ScalarArray_testConvert()
{
	printf("ScalarArray_testConvert\n");

	ScalarArray_t	sa;
	ScalarArray_t	sb;

	size_t count = 4000;

	sa.type = SCALAR_FLOAT32;
	sb.type = SCALAR_INT8;

	sb.count = sa.count = count;
	sb.vectordim = sa.vectordim = 1;

	sa.stride = sa.vectordim;
	sb.stride = sb.vectordim ;

	sa.data.tfloat = new float[count];
	sb.data.tint8 = new int8[count];

	for (size_t i = 0; i < sa.count; i++){
		sa.data.tfloat[i] = (frand()-0.5f)*255.0f;
	}

	int8 bmin = -127;
	int8 bmax = 127;
	float amin = -128.0f;
	float amax =  128.0f;


	double time = -getMicros();

	//ScalarArray_convertRanged(&sb,&bmin,&bmax,&sa,&amin,&amax);
	//ScalarArray_convert(&sb,&sa);

	time += getMicros();

	for (size_t i = 0; i < sb.count; i++){
		//printf("%i\n",sb.data.tint8[i]);
	}
	printf(" Time %.4f ms\n",time*1000.0);
}

void	ScalarArray_testSSE()
{
	printf("ScalarArray_testSSE\n");

	ScalarArray_t	sa;
	ScalarArray_t	sb;

	size_t count = 4000;

	sa.type = SCALAR_FLOAT32;
	sb.type = SCALAR_FLOAT32;

	sb.count = sa.count = count;
	sb.vectordim = sa.vectordim = 1;

	sa.stride = sa.vectordim;
	sb.stride = sb.vectordim ;

	float* saorig =	sa.data.tfloat = (float*)alignedMalloc(sizeof(float)*(sa.count+1),16);
	float* sborig = sb.data.tfloat = (float*)alignedMalloc(sizeof(float)*(sb.count+1),16);

	for (size_t i = 0; i < sa.count; i++){
		sa.data.tfloat[i] = 1.0f;
		sb.data.tfloat[i] = (frand()-0.5f)*255.0f;
	}


	volatile double time1 = 0.0;
	volatile double time2 = 0.0;

	int runs = 10000;
	while(runs--){
		// unaligned
		sa.data.tfloat = saorig+1;
		sb.data.tfloat = sborig+1;

		time1 -= getMicros();

		ScalarArray_Op2(&sa,SCALAR_OP2_MUL,&sa,&sb);

		time1 += getMicros();

		for (size_t i = 0; i < sb.count; i++){
			//printf("%.2f\n",sb.data.tfloat[i]-sa.data.tfloat[i]);
		}
		//printf(" Time %.4f ms\n",time1*1000.0);

		// aligned
		sa.data.tfloat = saorig;
		sb.data.tfloat = sborig;

		time2 -= getMicros();

		ScalarArray_Op2(&sa,SCALAR_OP2_MUL,&sa,&sb);

#if 0
		__m128  *o = (__m128*) sa.data.tfloat;
		__m128  *a = (__m128*) sa.data.tfloat;
		__m128  *b = (__m128*) sb.data.tfloat;
		for (size_t i = 0; i < count/4; i++, o++,a++,b++){
			*o = _mm_mul_ps(*a,*b);
		}
#endif
		time2 += getMicros();
	}

	printf("Time1: %.4f Time2: %.4f Time1/Time2 %.4f \n",time1*1000,time2*1000,time1/time2);
}


void Math_testVectorSSE()
{
	static Matrix44SIMD	matrix;

	printf("Math_testVectorSSE\n");
	
	Matrix44SSE_identity((Matrix44SSE)matrix);
	uint	cnt = 10000;

	matrix[12] = 1.0f;

	float*		vectorsA = (float*)alignedMalloc(sizeof(Vector4)*cnt,16);
	float*		vectorsB = (float*)alignedMalloc(sizeof(Vector4)*cnt,16);
	float*		vectorsC = (float*)alignedMalloc(sizeof(Vector4)*cnt,16);

	for (size_t i = 0; i < cnt*4; i++){
		if ((i+1) % 4 == 0){
			vectorsA[i] = 1.0f;
			vectorsB[i] = 1.0f;
			vectorsC[i] = 1.0f;
		}
		else{
			vectorsA[i] = 0.0f;
			vectorsB[i] = 0.0f;
			vectorsC[i] = 0.0f;
		}
	}

	double time1 = 0.0f;
	double time2 = 0.0f;

	uint runs = 100000;
	while (runs--){

		float	*vA = vectorsA;
		float	*vB = vectorsB;
		float	*vC = vectorsC;

		time1 -= getMicros();

		for (uint i = 0; i < cnt; i++, vA+=4, vB+=4, vC+=4){
			//Vector3Transform(vA,vB,matrix);

			//Vector3Add(vA,vB,vC);
			Vector3Mul(vA,vB,vC);
		}

		time1 += getMicros();


		vA = vectorsA;
		vB = vectorsB;
		vC = vectorsC;

		time2 -= getMicros();

		for (uint i = 0; i < cnt; i++, vA+=4, vB+=4, vC+=4){
			//Vector3SSE_transform((Vector3SSE)vA,(Vector3SSE)vB,(Matrix44SSE)matrix);
			//Vector4SSE_transformV3((Vector4SSE)vA,(Vector4SSE)vB,(Matrix44SSE)matrix);
			
			//Vector3SSE_add((Vector3SSE)vA,(Vector3SSE)vB,(Vector3SSE)vC);
			Vector3SSE_mul((Vector3SSE)vA,(Vector3SSE)vB,(Vector3SSE)vC);
		}

		time2 += getMicros();
	}
	printf("Time1: %.4f Time2: %.4f Time1/Time2 %.4f \n",time1*1000,time2*1000,time1/time2);
}

void Math_testMatrixSSE()
{
	printf("Math_testMatrixSSE\n");

	uint	cnt = 10000;
	uint	stride = 16;

	float*		vectorsA = (float*)alignedMalloc(sizeof(Matrix44)*cnt,16);
	float*		vectorsB = (float*)alignedMalloc(sizeof(Matrix44)*cnt,16);
	float*		vectorsC = (float*)alignedMalloc(sizeof(Matrix44)*cnt,16);

	for (size_t i = 0; i < cnt*stride; i++){
		vectorsA[i] = frand();
		vectorsB[i] = frand();
		vectorsC[i] = frand();
	}

	double time1 = 0.0f;
	double time2 = 0.0f;

	uint runs = 10000;
	while (runs--){

		float	*vA = vectorsA;
		float	*vB = vectorsB;
		float	*vC = vectorsC;

		time1 -= getMicros();

		for (uint i = 0; i < cnt; i++, vA+=stride, vB+=stride, vC+=stride){
			
			//Matrix44MultiplyFull(vC,vA,vB);
			//Matrix44Multiply(vC,vA,vB);

			//Matrix44Identity(vC);
			//Matrix44AffineInvert(vC,vA);
			//Matrix44Transpose1(vC);
			//Matrix44Copy(vC,vA);
			Matrix34TMultiply44(vC,vA,vB);
		}

		time1 += getMicros();


		vA = vectorsA;
		vB = vectorsB;
		vC = vectorsC;

		time2 -= getMicros();

		for (uint i = 0; i < cnt; i++, vA+=stride, vB+=stride, vC+=stride){
			
			//Matrix44SSE_multiply((Matrix44SSE)vC,(Matrix44SSE)vA,(Matrix44SSE)vB);

			//Matrix44SSE_identity((Matrix44SSE)vC);
			//Matrix44SSE_affineInvert((Matrix44SSE)vC,(Matrix44SSE)vA);
			//Matrix44SSE_transpose1((Matrix44SSE)vC);
			//Matrix44SSE_copy((Matrix44SSE)vC,(Matrix44SSE)vA);
			Matrix34SSE_tmultiply44((Matrix44SSE)vC,(Matrix44SSE)vA,(Matrix44SSE)vB);
		}

		time2 += getMicros();
	}

	printf("Time1: %.4f Time2: %.4f Time1/Time2 %.4f \n",time1*1000,time2*1000,time1/time2);
}

void floatbits()
{

	float x;
	scanf("%g", &x);
	printf("%08X\n", (*((int *)(&x))));
}

#define ITEROPS	8

typedef struct IterValue_s{
	int			a;
	int			b;
	int			c;
	int			somedata[32];
}IterValue_t;

typedef struct IterTest1_s{
	int			numops;
	int			ops[ITEROPS];
	IterValue_t	val;
}IterTest1_t;

typedef struct IterTest2_s{
	int			flag;
	IterValue_t	val;
}IterTest2_t;

typedef struct IterTest3_s{
	int			ops[ITEROPS];
	IterValue_t	val;
}IterTest3_t;

typedef struct IterTest4_s{
	byte		ops[ITEROPS];
	IterValue_t	val;
}IterTest4_t;


LUX_INLINE void itop0(IterValue_t &val)
{
	val.c = val.a + val.b;
}
LUX_INLINE void itop1(IterValue_t &val)
{
	val.c = val.a - val.b;
}
LUX_INLINE void itop2(IterValue_t &val)
{
	val.a = val.c * val.b;
}
LUX_INLINE void itop3(IterValue_t &val)
{
	val.b = val.a*val.a + val.c;
}


void iteratortest()
{
	const int count = 8192;
	const int runs = 32;
	IterTest1_t*	data1 = new IterTest1_t[count];
	IterTest2_t*	data2 = new IterTest2_t[count];
	IterTest3_t*	data3 = new IterTest3_t[count];
	IterTest4_t*	data4 = new IterTest4_t[count];
	
	

	for (int i = 0; i < count; i++){
		IterTest1_t *it = &data1[i];
		IterTest2_t *it2 = &data2[i];
		IterTest3_t *it3 = &data3[i];
		IterTest4_t *it4 = &data4[i];

		it->val.a = rand();
		it->val.b = rand();
		it->val.c = rand();
		
		it2->val = it->val;
		it3->val = it->val;
		it4->val = it->val;

		it->numops = rand()%ITEROPS;

		memset(it3->ops,0,sizeof(it3->ops));
		memset(it4->ops,0,sizeof(it4->ops));

		for(int n = 0; n < it->numops; n++){
			int op = n;
			it->ops[n] = op;
			it2->flag |= 1<<op;
			it3->ops[op] = 1;
			it4->ops[op] = 1;
		}
		
	}
#if 1
	// test1
	double time1 = 0;
	for (int r = 0; r < runs; r++){
		time1 -= getMicros();
		for (int i = 0; i < count; i++){
			IterTest1_t *it = &data1[i];
			int o = it->numops;
			while (o--){
				switch(it->ops[o]){
				case 0: itop0(it->val); break;
				case 1: itop1(it->val); break;
				case 2: itop2(it->val); break;
				case 3: itop3(it->val); break;

				case 4: itop0(it->val); break;
				case 5: itop1(it->val); break;
				case 6: itop2(it->val); break;
				case 7: itop3(it->val); break;
				default:
					LUX_ASSUME(0); break;
				}
			}

		}
		time1 += getMicros();
	}

	double time2 = 0;
	for (int r = 0; r < runs; r++){
		time2 -= getMicros();
		for (int i = 0; i < count; i++){
			IterTest2_t *it = &data2[i];
			if (it->flag & 1<<0) itop0(it->val);
			if (it->flag & 1<<1) itop1(it->val);
			if (it->flag & 1<<2) itop2(it->val);
			if (it->flag & 1<<3) itop3(it->val);

			if (it->flag & 1<<4) itop0(it->val);
			if (it->flag & 1<<5) itop1(it->val);
			if (it->flag & 1<<6) itop2(it->val);
			if (it->flag & 1<<7) itop3(it->val);
		}
		time2 += getMicros();
	}


	double time3 = 0;
	for (int r = 0; r < runs; r++){
		time3 -= getMicros();
		for (int i = 0; i < count; i++){
			IterTest3_t *it = &data3[i];
			if (it->ops[0]) itop0(it->val);
			if (it->ops[1]) itop1(it->val);
			if (it->ops[2]) itop2(it->val);
			if (it->ops[3]) itop3(it->val);

			if (it->ops[4]) itop0(it->val);
			if (it->ops[5]) itop1(it->val);
			if (it->ops[6]) itop2(it->val);
			if (it->ops[7]) itop3(it->val);
		}
		time3 += getMicros();
	}

	double time4 = 0;
	for (int r = 0; r < runs; r++){
		time4 -= getMicros();
		for (int i = 0; i < count; i++){
			IterTest4_t *it = &data4[i];
			if (it->ops[0]) itop0(it->val);
			if (it->ops[1]) itop1(it->val);
			if (it->ops[2]) itop2(it->val);
			if (it->ops[3]) itop3(it->val);

			if (it->ops[4]) itop0(it->val);
			if (it->ops[5]) itop1(it->val);
			if (it->ops[6]) itop2(it->val);
			if (it->ops[7]) itop3(it->val);
		}
		time4 += getMicros();
	}


	printf("Time1: %.4f\n",time1*1000);
	printf("Time2: %.4f\n",time2*1000);
	printf("Time3: %.4f\n",time3*1000);
	printf("Time4: %.4f\n",time4*1000);

#endif
	printf("Change Test\n");
	// new test

	double time5 = 0;
	for (int r = 0; r < runs; r++){
		time5 -= getMicros();
		int curflag = 0;
		for (int i = 0; i < count; i++){
			IterTest2_t *it = &data2[i];
			int changed = it->flag ^ curflag;
			for (int n = 0; n < ITEROPS; n++){
				if (changed & 1<<n){
					if (it->flag & 1<<0) itop0(it->val);
					else				itop1(it->val);

					if (it->flag & 1<<1) itop1(it->val);
					else				itop2(it->val);

					if (it->flag & 1<<2) itop2(it->val);
					else				itop3(it->val);

					if (it->flag & 1<<3) itop3(it->val);
					else				itop0(it->val);

					if (it->flag & 1<<4) itop0(it->val);
					else				itop1(it->val);

					if (it->flag & 1<<5) itop1(it->val);
					else				itop2(it->val);

					if (it->flag & 1<<6) itop2(it->val);
					else				itop3(it->val);

					if (it->flag & 1<<7) itop3(it->val);
					else				itop0(it->val);



					if (it->flag & 1<<8) itop0(it->val);
					else				itop1(it->val);

					if (it->flag & 1<<9) itop1(it->val);
					else				itop2(it->val);

					if (it->flag & 1<<10) itop2(it->val);
					else				itop3(it->val);

					if (it->flag & 1<<11) itop3(it->val);
					else				itop0(it->val);

					if (it->flag & 1<<12) itop0(it->val);
					else				itop1(it->val);

					if (it->flag & 1<<13) itop1(it->val);
					else				itop2(it->val);

					if (it->flag & 1<<14) itop2(it->val);
					else				itop3(it->val);

					if (it->flag & 1<<15) itop3(it->val);
					else				itop0(it->val);
				}
			}

			curflag = it->flag;
		}
		time5 += getMicros();
	}

	double time6 = 0;
	for (int r = 0; r < runs; r++){
		time6 -= getMicros();
		int curflag = 0;
		for (int i = 0; i < count; i++){
			IterTest2_t *it = &data2[i];
			int changed = it->flag ^ curflag;
				if (changed & 1<<0){
					if (it->flag & 1<<0) itop0(it->val);
					else				itop1(it->val);
				}
				if (changed & 1<<1){
					if (it->flag & 1<<1) itop1(it->val);
					else				itop2(it->val);
				}
				if (changed & 1<<2){
					if (it->flag & 1<<2) itop2(it->val);
					else				itop3(it->val);
				}
				if (changed & 1<<3){
					if (it->flag & 1<<3) itop3(it->val);
					else				itop0(it->val);
				}
				if (changed & 1<<4){
					if (it->flag & 1<<4) itop0(it->val);
					else				itop1(it->val);
				}
				if (changed & 1<<5){
					if (it->flag & 1<<5) itop1(it->val);
					else				itop2(it->val);
				}
				if (changed & 1<<6){
					if (it->flag & 1<<6) itop2(it->val);
					else				itop3(it->val);
				}
				if (changed & 1<<7){
					if (it->flag & 1<<7) itop3(it->val);
					else				itop0(it->val);
				}


				if (changed & 1<<8){
					if (it->flag & 1<<8) itop0(it->val);
					else				itop1(it->val);
				}
				if (changed & 1<<1){
					if (it->flag & 1<<9) itop1(it->val);
					else				itop2(it->val);
				}
				if (changed & 1<<10){
					if (it->flag & 1<<10) itop2(it->val);
					else				itop3(it->val);
				}
				if (changed & 1<<11){
					if (it->flag & 1<<11) itop3(it->val);
					else				itop0(it->val);
				}
				if (changed & 1<<12){
					if (it->flag & 1<<12) itop0(it->val);
					else				itop1(it->val);
				}
				if (changed & 1<<13){
					if (it->flag & 1<<13) itop1(it->val);
					else				itop2(it->val);
				}
				if (changed & 1<<14){
					if (it->flag & 1<<14) itop2(it->val);
					else				itop3(it->val);
				}
				if (changed & 1<<15){
					if (it->flag & 1<<15) itop3(it->val);
					else				itop0(it->val);
				}

			curflag = it->flag;
		}
		time6 += getMicros();
	}

	double time7 = 0;
	for (int r = 0; r < runs; r++){
		time7 -= getMicros();
		int curflag = 0;
		for (int i = 0; i < count; i++){
			IterTest2_t *it = &data2[i];
			int changed = it->flag ^ curflag;
			int enable = changed & it->flag;
			int disable = changed & curflag;

			
			if (enable & 1<<0) itop0(it->val);
			if( disable & 1<<0) itop1(it->val);

			if (enable & 1<<1) itop1(it->val);
			if( disable & 1<<1) itop2(it->val);

			if (enable & 1<<2) itop2(it->val);
			if( disable & 1<<2) itop3(it->val);

			if (enable & 1<<3) itop3(it->val);
			if( disable & 1<<3) itop0(it->val);

			if (enable & 1<<4) itop0(it->val);
			if( disable & 1<<4) itop1(it->val);

			if (enable & 1<<5) itop1(it->val);
			if( disable & 1<<5) itop2(it->val);

			if (enable & 1<<6) itop2(it->val);
			if( disable & 1<<6) itop3(it->val);

			if (enable & 1<<7) itop3(it->val);
			if( disable & 1<<7) itop0(it->val);



			if (enable & 1<<8) itop0(it->val);
			if( disable & 1<<8) itop1(it->val);

			if (enable & 1<<9) itop1(it->val);
			if( disable & 1<<9) itop2(it->val);

			if (enable & 1<<10) itop2(it->val);
			if( disable & 1<<10) itop3(it->val);

			if (enable & 1<<11) itop3(it->val);
			if( disable & 1<<11) itop0(it->val);

			if (enable & 1<<12) itop0(it->val);
			if( disable & 1<<12) itop1(it->val);

			if (enable & 1<<13) itop1(it->val);
			if( disable & 1<<13) itop2(it->val);

			if (enable & 1<<14) itop2(it->val);
			if( disable & 1<<14) itop3(it->val);

			if (enable & 1<<15) itop3(it->val);
			if( disable & 1<<15) itop0(it->val);

			curflag = it->flag;
		}
		time7 += getMicros();
	}
	

	printf("Time5: %.4f\n",time5*1000);
	printf("Time6: %.4f\n",time6*1000);
	printf("Time7: %.4f\n",time7*1000);
}

int main(int argc, char **argv)
{
	initTimer();

	//ScalarArray_testConvert();
	//ScalarArray_testSSE();

	//Math_testVectorSSE();
	//Math_testMatrixSSE();

	//floatbits();
	iteratortest();

	return 0;
}

