// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt
//
// <Author: Christoph Kubisch>


#ifndef __LUXMATH_BOUNDING_H__
#define __LUXMATH_BOUDNING_H__

#include <luxmath/basetypes.h>

#ifdef __cplusplus
extern "C"{
#endif


//////////////////////////////////////////////////////////////////////////
// Generic

booln minmaxIntersects(float selfminmax[6], float otherminmax[6]);


///////////////////////////////////////////////////////////////////////////////
// BOUNDING VOLUMES

void BoundingBox_init(BoundingBox_t *box);

void BoundingBox_complete(BoundingBox_t *box, booln fromMinMax);

// copy a bounding volume to another
BoundingBox_t *BoundingBox_copy(BoundingBox_t *out, BoundingBox_t *in);
BoundingSphere_t *BoundingSphere_copy(BoundingSphere_t *out, const BoundingSphere_t *in);


// use min/max
// ------------
// merge two bounding volumes
BoundingBox_t * BoundingBox_merge(BoundingBox_t *out, BoundingBox_t *a, BoundingBox_t *b);
// returns TRUE if result differs from a
booln	BoundingBox_mergeChange(BoundingBox_t *out, BoundingBox_t *a, BoundingBox_t *b);
booln	BoundingSphere_mergeChange(BoundingSphere_t *out, const BoundingSphere_t *a, const BoundingSphere_t *b);

// converts a bounding box to a bounding sphere
BoundingSphere_t *BoundingBox_toSphere(BoundingBox_t *bbox, BoundingSphere_t *sphere);
void BoundingBox_toSphereV(const Vector3 min,const Vector3 max, Vector3 center, float *radius);

// convert bounding box to capsule
// a capsule along major axis of boundingbox is generated with radius
// of the greater half dimension of the other 2 axis
BoundingCapsule_t *BoundingBox_toCapsule(BoundingBox_t *bbox,BoundingCapsule_t *capsule);

// transforms a bounding box to an axis aligned bounding box
BoundingBox_t *BoundingBox_transform( BoundingBox_t *out, BoundingBox_t *in, Matrix44 trans);
void BoundingBox_transformBoxCorners(const BoundingBox_t *in, const Matrix44 trans, Vector3 box[8]);
void BoundingBox_transformV(Vector3 outmins, Vector3 outmaxs,Vector3 mins, Vector3 maxs, Matrix44 trans);

void BoundingBox_fromCorners(BoundingBox_t *bbox,const Vector3 vecs[8]);
void BoundingVectors_fromCamera(Vector3 vecs[8],const Matrix44 mat,const float fov, const float frontplane, const float backplane, const float aspect);

booln BoundingBox_intersect(const BoundingBox_t *a,const BoundingBox_t *b);
booln	BoundingBox_checkPoint(const BoundingBox_t *out,const Vector3 point);
booln BoundingCone_checkSphereV(BoundingCone_t *cone,Vector3 center, float radius, float radiusSqr);

// approximates frustum via Sphere & Cone, dir must be normalized
void BoundingSphereCone_fromCamera(BoundingSphere_t *sphere, BoundingCone_t *cone,float frontplane, float backplane, Vector3 pos, Vector3 dir, float fov);
void BoundingCone_fromFrustumCorners(BoundingCone_t *cone, Vector3 box[FRUSTUM_CORNERS]);
void BoundingSphere_fromFrustumCorners(BoundingSphere_t *sphere,Vector3 box[FRUSTUM_CORNERS]);

//////////////////////////////////////////////////////////////////////////
LUX_INLINE void BoundingBox_init(BoundingBox_t *box)
{
	box->min[0] = 0.0f;
	box->min[1] = 0.0f;
	box->min[2] = 0.0f;

	box->max[0] = 0.0f;
	box->max[1] = 0.0f;
	box->max[2] = 0.0f;

	box->center[0] = 0.0f;
	box->center[1] = 0.0f;
	box->center[2] = 0.0f;

	box->length[0] = 0.0f;
	box->length[1] = 0.0f;
	box->length[2] = 0.0f;
}

LUX_INLINE booln minmaxIntersects(float selfminmax[6], float otherminmax[6]){
	return (	
		selfminmax[0] <= otherminmax[3] && 
		selfminmax[1] <= otherminmax[4] &&
		selfminmax[2] <= otherminmax[5] &&
		selfminmax[3] >= otherminmax[0] &&
		selfminmax[4] >= otherminmax[1] &&
		selfminmax[5] >= otherminmax[2]	);
}

#ifdef __cplusplus
}
#endif

#endif

