// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt
//
// <Author: Christoph Kubisch>


#ifndef __LUXMATH_SIMD_H__
#define __LUXMATH_SIMD_H__

#include <luxmath/basetypes.h>

#ifdef __cplusplus
extern "C"{
#endif


//////////////////////////////////////////////////////////////////////////
// Math SIMD
//
// A few wrapper macros, that either forward to regular
// math functions, or SIMD version.
// There can be only one SIMD version active.

#if defined(LUX_SIMD_SSE)
// Only use when you are sure that pointers passed are 16-byte aligned
#include <luxmath/xmmmath.h>
#define Matrix44IdentitySIMD(o)			Matrix44SSE_identity((__m128*)(o))
#define Matrix44Transpose1SIMD(o)		Matrix44SSE_transpose1((__m128*)(o))
#define Matrix44CopySIMD(o,a)			Matrix44SSE_copy((__m128*)(o),(__m128*)(a))
#define Matrix44AffineInvertSIMD(o,a)	Matrix44SSE_affineInvert((__m128*)(o),(__m128*)(a))
#define Matrix44InvertSIMD(o,a)			Matrix44SSE_invert((__m128*)(o),(__m128*)(a))
#define Matrix44TransposeSIMD(o,a)		Matrix44SSE_transpose((__m128*)(o),(__m128*)(a))
#define Matrix44Multiply1SIMD(o,a)		Matrix44SSE_multiply1((__m128*)(o),(__m128*)(a))
#define Matrix44Multiply2SIMD(a,o)		Matrix44SSE_multiply2((__m128*)(a),(__m128*)(o))
#define Matrix44MultiplySIMD(o,a,b)		Matrix44SSE_multiply((__m128*)(o),(__m128*)(a),(__m128*)(b))
#define Matrix44MultiplyFullSIMD(o,a,b)	Matrix44SSE_multiply((__m128*)(o),(__m128*)(a),(__m128*)(b))


#define Matrix34IdentitySIMD(o)			Matrix34SSE_identity((__m128*)(o))
#define Matrix34CopySIMD(o,a)			Matrix34SSE_copy((__m128*)(o),(__m128*)(a))
#define Matrix34TMultiply44SIMD(o,a,b)	Matrix34SSE_tmultiply44((__m128*)(o),(__m128*)(a),(__m128*)(b))

#define Vector4LerpSIMD(o,t,a,b)			Vector4SSE_lerp((__m128*)(o),t,(__m128*)(a),(__m128*)(b))
#define Vector4CatmullRomSIMD(o,t,a,b,c,d)	Vector4SSE_catmullRom((__m128*)(o),t,(__m128*)(a),(__m128*)(b),(__m128*)(c),(__m128*)(d))
#else
#include <luxmath/matrix44.h>
#define Matrix44IdentitySIMD			Matrix44Identity
#define Matrix44Transpose1SIMD			Matrix44Transpose1
#define Matrix44CopySIMD				Matrix44Copy
#define Matrix44AffineInvertSIMD		Matrix44AffineInvert
#define Matrix44InvertSIMD				Matrix44Invert
#define Matrix44TransposeSIMD			Matrix44Transpose
#define Matrix44Multiply1SIMD			Matrix44Multiply1
#define Matrix44Multiply2SIMD			Matrix44Multiply2
#define Matrix44MultiplySIMD			Matrix44Multiply
#define Matrix44MultiplyFullSIMD		Matrix44MultiplyFull

#define Matrix34IdentitySIMD			Matrix34Identity
#define Matrix34CopySIMD				Matrix34Copy
#define Matrix34TMultiply44SIMD			Matrix34TMultiply44

#define Vector4LerpSIMD(o,t,a,b)		Vector4Lerp
#define Vector4CatmullRomSIMD			Vector4CatmullRom
#endif

#ifdef __cplusplus
}
#endif

#endif
