// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt

#include <luxmath/geometry.h>
#include <luxmath/vector3.h>
#include <luxmath/vector4.h>
#include <luxmath/matrix44.h>

void PlaneSet
(Plane plane, const Vector3 a, const Vector3 b, const Vector3 c)
{
	Vector3 toB;
	Vector3 toC;

	Vector3Sub(toB,b,a);
	Vector3Sub(toC,c,a);
	Vector3Cross(plane,toB,toC);
	Vector3NormalizedA(plane);
	plane[3] = -Vector3Dot(plane,a);
}

void PlaneIntersect
(Vector3 out, const Plane planea,const Plane planeb,const Plane planec)
{
	// http://local.wasp.uwa.edu.au/~pbourke/geometry/3planes/
	Vector3 da;
	Vector3 db;
	Vector3 dc;

	Vector3Cross(da,planec,planeb);
	Vector3Cross(db,planea,planec);
	Vector3Cross(dc,planeb,planea);

	Vector3Scale(dc,dc,-planec[3]);
	Vector3ScaledAdd(dc,dc,-planeb[3],db);
	Vector3ScaledAdd(dc,dc,-planea[3],da);

	Vector3Scale(out,dc,1.0f/(Vector3Dot(planea,da)));
}

void PlaneVecCheckLine
(Vector3 intersectpoint, const Vector3 planenormal, const Vector3 planepoint, const Vector3 raystart, const Vector3 rayend)
{
	Vector3 dir;
	float ratio;
	float distp = Vector3Dot(planenormal,planepoint);
	float diststart = Vector3Dot(planenormal,raystart);
	float distend = Vector3Dot(planenormal,rayend);

	ratio = (distp-diststart)/(distend-diststart);
	ratio = MIN(ratio,1.0f);
	ratio = MAX(ratio,0.0f);
	Vector3Sub(dir,rayend,raystart);
	Vector3ScaledAdd(intersectpoint,raystart,ratio,dir);
}

float TriangleArea
( const Vector3 v0, const Vector3 v1, const Vector3 v2)
{
	Vector3 a,b,cross;
	Vector3Sub(a,v1,v0);
	Vector3Sub(b,v2,v0);
	Vector3Cross(cross,a,b);

	return Vector3Length(cross)*0.5f;
}
