// Copyright (C) 2004-2009 Christoph Kubisch & Eike Decker
// This file is part of the "Luxinia Engine".
// For conditions of distribution and use, see copyright notice in luxiniapublicsdk.txt

#include <luxmath/vector3.h>
#include <luxmath/misc.h>

float Vector3LineDistanceSq(const Vector3 point,const Vector3 linestart,const Vector3 linedir)
{
	Vector3	topoint;
	float	fracc;
	float	sqlen;

	Vector3Sub(topoint,point,linestart);

	fracc = Vector3Dot(linedir,topoint);

	if (fracc > 0){
		sqlen = Vector3SqLength(linedir);
		if (fracc >= sqlen)
			Vector3Sub(topoint,topoint,linedir);
		else{
			fracc /= sqlen;
			Vector3ScaledAdd(topoint,topoint,-fracc,linedir);
		}
	}

	return Vector3SqLength(topoint);
}
float Vector3LineDistanceSqFracc(const Vector3 point,const Vector3 linestart, const Vector3 linedir, float *ofracc)
{
	Vector3	topoint;
	float	fracc;
	float	sqlen;

	Vector3Sub(topoint,point,linestart);

	fracc = Vector3Dot(linedir,topoint);

	if (fracc > 0){
		sqlen = Vector3SqLength(linedir);
		if (fracc >= sqlen)
			Vector3Sub(topoint,topoint,linedir);
		else{
			fracc /= sqlen;
			Vector3ScaledAdd(topoint,topoint,-fracc,linedir);
		}
	}
	*ofracc = MIN(1,MAX(0,fracc));

	return Vector3SqLength(topoint);
}

void Vector3Spread(Vector3 out, Vector3 in, const float radin, const float radout){
	Vector3 forward;
	Vector3 right;
	Vector3 up;
	float length;
	float spread;
	float	angle;
	uchar	flip;

	flip = FALSE;
	Vector3Copy(forward,in);
	length = Vector3Normalized(forward);
	Vector3Perpendicular(right,forward);
	Vector3Cross(up,forward,right);

	spread = radin + frand()*(radout-radin);
	if (spread > MUL_HALF_PI)
		length = -length;
	spread = fastsin(spread)/fastcos(spread); // (float)tan (spread);
	angle = frand()*MUL_TWOPI;

	Vector3Copy(out,forward);
	Vector3ScaledAdd(out,out,spread*fastsin(angle),right);
	Vector3ScaledAdd(out,out,spread*fastcos(angle),up);

	Vector3Normalized(out);
	Vector3Scale(out,out,length);
}

void Vector3Perpendicular(Vector3 out, Vector3 in)
{	// creates a random normalized vector which is perpendicular to in
	Vector3 nin;
	Vector3 x = {1.0f,0.0f,0.0f};
	Vector3 y = {0.0f,1.0f,0.0f};
	Vector3 z = {0.0f,0.0f,1.0f};
	float rdot;
	// ~25 tolerance
	Vector3Normalize(nin,in);
	rdot = Vector3Dot(x,nin);
	if (FP_ABS_BITS(rdot) < 0x3F666666){
		Vector3Cross(out,x,nin);
	}
	else{
		rdot = Vector3Dot(y,nin);
		if (FP_ABS_BITS(rdot) < 0x3F666666){
			Vector3Cross(out,z,nin);
		}
		else{
			Vector3Cross(out,y,nin);
		}
	}
	Vector3Normalized(out);
}

void Vector3PerpendicularA(Vector3 out, Vector3 in)
{	// creates a random normalized vector which is perpendicular to in
	Vector3 nin;
	Vector3 x = {1.0f,0.0f,0.0f};
	Vector3 y = {0.0f,1.0f,0.0f};
	Vector3 z = {0.0f,0.0f,1.0f};
	float rdot;
	// ~25 tolerance
	Vector3NormalizeA(nin,in);
	rdot = Vector3Dot(x,nin);
	if (FP_ABS_BITS(rdot) < 0x3F666666){
		Vector3Cross(out,x,nin);
	}
	else{
		rdot = Vector3Dot(y,nin);
		if (FP_ABS_BITS(rdot) < 0x3F666666){
			Vector3Cross(out,z,nin);
		}
		else{
			Vector3Cross(out,y,nin);
		}
	}
	Vector3NormalizedA(out);
}

void Vector3MinMax(Vector3 min, Vector3 max, Vector3 pos)
{
	min[0] = MIN(min[0],pos[0]);
	min[1] = MIN(min[1],pos[1]);
	min[2] = MIN(min[2],pos[2]);

	max[0] = MAX(max[0],pos[0]);
	max[1] = MAX(max[1],pos[1]);
	max[2] = MAX(max[2],pos[2]);
}
